/*
 * Copyright (C) 2004 Damien Marchal, Bernard Leach
 * Copyright (C) 2005 Courtney Cavin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/time.h>
#ifdef IPOD
#include <linux/vt.h>
#endif
#include <dirent.h>
#include <string.h>

#include "pz.h"
#include "ipod.h"
#include "mlist.h"
#include "piezo.h"

static GR_WINDOW_ID browser_wid;
static GR_GC_ID browser_gc;
static menu_st *browser_menu, *browser_menu_overlay;

#define FILE_TYPE_PROGRAM 0
#define FILE_TYPE_DIRECTORY 1
#define FILE_TYPE_OTHER 2
#define MAX_ENTRIES 256

typedef struct {
	char name[64];
	char *full_name;
	unsigned short type;
	unsigned int op;
} Directory;

static char current_dir[128];
static char *current_file;
static int browser_nbEntries = 0;
static Directory browser_entries[MAX_ENTRIES];

void new_exec_window(char *filename);

/* START JONRELAY'S PODZILLAE BROWSER CRAP */

static int is_a_podzilla(char * f)
{
	int i;
	for (i = strlen(f)-5; i>=0; i--) {
		if (!memcmp(f+i,"zilla",5)) { return (f[i+5] != 'e'); }
		if (!memcmp(f+i,"relon",5)) { return 1; }
	}
	return 0;
}

static int contains_string(char * s, char * c)
{
	int i;
	int l = strlen(c);
	for (i = strlen(s)-l; i>=0; i--) {
		if (!memcmp(s+i,c,l)) { return 1; }
	}
	return 0;
}

static char * find_most_recent(char *dira, char *c)
{
	DIR *dir = opendir(dira);
	struct stat stat_result;
	struct dirent *subdir;
	char * mrf = NULL;
	time_t mrt = 0;
	
	while ((subdir = readdir(dir))) {
		if(contains_string(subdir->d_name, c)) {
			stat(subdir->d_name, &stat_result);
			if (stat_result.st_mtime > mrt) {
				mrt = stat_result.st_mtime;
				mrf = strdup(subdir->d_name);
			}
		}
	}
	closedir(dir);
	return mrf;
}

void launch_most_recent(char * dira, char * c)
{
	char * mr;
	if ( (mr = find_most_recent(dira, c)) != NULL ) {
		new_exec_window(mr);
	}
}

void launch_podzilla(void) {
	char * mr;
	chdir("/sbin");
	if ( (mr = find_most_recent("./", "-podzilla")) != NULL ) {
		new_exec_window(mr);
	} else if ( (mr = find_most_recent("./", "podzilla-0")) != NULL ) {
		new_exec_window(mr);
	} else if ( (mr = find_most_recent("./", "podzilla-")) != NULL ) {
		new_exec_window(mr);
	} else if ( (mr = find_most_recent("./", "podzilla")) != NULL ) {
		new_exec_window(mr);
	}
}

void launch_vipodzilla(void) { chdir("/sbin"); launch_most_recent("./", "vipodzilla"); }
void launch_pokezilla(void) { chdir("/sbin"); launch_most_recent("./", "pokezilla"); }
void launch_floydzilla(void) { chdir("/sbin"); launch_most_recent("./", "floydzilla"); }
void launch_yankeezilla(void) { chdir("/sbin"); launch_most_recent("./", "yankeezilla"); }
void launch_podzillati(void) { chdir("/sbin"); launch_most_recent("./", "podzilla-ti"); }

/* END JONRELAY'S PODZILLAE BROWSER CRAP */

static int dir_cmp(const void *x, const void *y) 
{
	Directory *a = (Directory *)x;
	Directory *b = (Directory *)y;

	if (a->type == FILE_TYPE_DIRECTORY &&
		b->type != FILE_TYPE_DIRECTORY)
		return -1;
	if (a->type != FILE_TYPE_DIRECTORY &&
		b->type == FILE_TYPE_DIRECTORY)
		return 1;
	
	return strcasecmp(a->name, b->name);
} 

static void browser_exit()
{
	int i;

	for (i = 0; i < browser_nbEntries; i++) {
		free(browser_entries[i].full_name);
	}
	browser_nbEntries = 0;
}

/* the directory to scan */
static void browser_mscandir(char *dira)
{
	DIR *dir = opendir(dira);
	int i, size, op;
	struct stat stat_result;
	struct dirent *subdir;

	if(browser_menu != NULL) {
		menu_destroy(browser_menu);
	}
	browser_menu = menu_init(browser_wid, _("All Podzillae"), 0, 0,
			screen_info.cols, screen_info.rows -
			(HEADER_TOPLINE + 1), NULL, NULL, ASCII);

	/* not very good for fragmentation... */
	for (i = 0; i < browser_nbEntries; i++) {
		free(browser_entries[i].full_name);
	}
	browser_nbEntries = 0;

	while ((subdir = readdir(dir))) {
		if(strncmp(subdir->d_name, ".", strlen(subdir->d_name)) == 0) {
			continue;
		}
		if(strncmp(subdir->d_name, "..", strlen(subdir->d_name)) == 0) {
			strcpy(browser_entries[browser_nbEntries].name,
				_(".. [Parent Directory]"));
			browser_entries[browser_nbEntries].full_name =
				strdup(browser_entries[browser_nbEntries].name);
			browser_entries[browser_nbEntries].type =
				FILE_TYPE_DIRECTORY;
			browser_entries[browser_nbEntries].op = STUB_MENU;
			browser_nbEntries++;
			continue;
		}
		if(!ipod_get_setting(BROWSER_HIDDEN)) {
			if (subdir->d_name[0] == '.') {
				continue;
			}
		}
		stat(subdir->d_name, &stat_result);
		if(!S_ISDIR(stat_result.st_mode)) {
			if(!is_a_podzilla(subdir->d_name)) {
				continue;
			}
		}
		if(browser_nbEntries >= MAX_ENTRIES) {
			pz_error(_("Directory contains too many files."));
			break;
		}
		
		size = strlen(subdir->d_name);
		size = (size > 62 ? 62 : size);
		browser_entries[browser_nbEntries].full_name =
			strdup(subdir->d_name);

		strncpy(browser_entries[browser_nbEntries].name,
				subdir->d_name, size);
		browser_entries[browser_nbEntries].name[size] = '\0';

		stat(subdir->d_name, &stat_result);
		if(S_ISDIR(stat_result.st_mode)) {
			browser_entries[browser_nbEntries].type =
			    FILE_TYPE_DIRECTORY;
			op = ARROW_MENU;
		}
		else if(stat_result.st_mode & S_IXUSR) {
			browser_entries[browser_nbEntries].type =
			    FILE_TYPE_PROGRAM;
			op = EXECUTE_MENU;
		}
		else {
			browser_entries[browser_nbEntries].type =
				FILE_TYPE_OTHER;
			op = STUB_MENU;
		}
		browser_entries[browser_nbEntries].op = op;

		browser_nbEntries++;
	}
	closedir(dir);

	qsort(browser_entries, browser_nbEntries, sizeof(Directory), dir_cmp);
	for (i = 0; i < browser_nbEntries; i++)
		menu_add_item(browser_menu, browser_entries[i].name,
				NULL, 0, browser_entries[i].op);
}

static void browser_exec_file(char *filename)
{
	int len;
	char *path;

	len = strlen(filename) + strlen(current_dir) + 2;
	path = (char *)malloc(len * sizeof(char));
	snprintf(path, len, "%s/%s", current_dir, filename);
	new_exec_window(path);
	free(path);
}

static char short_current_dir[80];
static char * browser_shorten_path()
{
	int rlen = strlen( current_dir );
	int pos;
	int max = 20;

	/* this following block is untested... */
	if( screen_info.cols > 160 ) /* photo */
		max = 35;
	else if( screen_info.cols < 160 ) /* mini */
		max = 17;

	if( rlen > 20 ) /* 20 is the cutoff point */
	{
		strcpy( short_current_dir, "--" );
		for( pos=2; pos<(20+2) ; pos++ )
		{
			short_current_dir[pos] = current_dir[rlen-20+pos];
		}
		short_current_dir[pos] = '\0';
	} else {
		strcpy( short_current_dir, current_dir );
	}
	return( short_current_dir );
}

static void browser_do_draw()
{
	/* window is focused */
	if(browser_wid == GrGetFocus()) {
		if( ipod_get_setting( BROWSER_PATH ))
			pz_draw_header(browser_shorten_path());
		else
			pz_draw_header(browser_menu->title);
		menu_draw(browser_menu);
	}
}

static int is_script_type(char *extension)
{
	return strcmp(extension, ".sh") == 0;
}

static int is_binary_type(char *filename)
{
	FILE *fp;
	unsigned char header[12];
	struct stat ftype; 

	stat(filename, &ftype); 
	if(S_ISBLK(ftype.st_mode)||S_ISCHR(ftype.st_mode))
		return 0;
	if((fp = fopen(filename, "r"))==NULL) {
		perror(filename);
		return 0;
	}

	fread(header, sizeof(char), 12, fp);
	
	fclose(fp);
	if(strncmp((const char *)header, "bFLT", 4)==0)
		return 1;
	if(strncmp((const char *)header, "#!/bin/sh", 9)==0)
		return 1;
	return 0;
}

static void handle_type_other(char *filename)
{
	char *ext;

	ext = strrchr(filename, '.');
	if (ext == 0) {
		if (is_binary_type(filename)) {
			browser_exec_file(filename);
		}
		else {
			new_message_window(_("Not an executable file."));
		}
	}
	else if (is_script_type(ext)) {
		browser_exec_file(filename);
	}
	else if (is_binary_type(filename)) {
		browser_exec_file(filename);
	}
	else  {
		new_message_window(_("Not an executable file."));
	}
}

static void browser_selection_activated(unsigned short userChoice)
{
	if (userChoice == 0) {
		chdir("..");
		getcwd(current_dir, sizeof(current_dir));
		browser_mscandir("./");
	}
	switch (browser_entries[userChoice].type) {
	case FILE_TYPE_DIRECTORY:
		chdir(browser_entries[userChoice].full_name);
		getcwd(current_dir, sizeof(current_dir));
		browser_mscandir("./");
		break;
	case FILE_TYPE_PROGRAM:
	case FILE_TYPE_OTHER:
		handle_type_other(browser_entries[userChoice].full_name);
		break;
	}
}

static int browser_do_keystroke(GR_EVENT * event)
{
	int ret = 0;

	switch(event->type) {
	case GR_EVENT_TYPE_TIMER:
		menu_draw_timer(browser_menu);
		break;

	case GR_EVENT_TYPE_KEY_DOWN:
		switch (event->keystroke.ch) {
		case '\r':
		case '\n':
			if(browser_menu->parent == NULL) {
				menu_handle_timer(browser_menu, 1);
				browser_selection_activated(browser_menu->items[browser_menu->sel].orig_pos);
				browser_do_draw();
			} else {
				menu_handle_timer(browser_menu, 1);
				browser_menu = menu_handle_item(browser_menu, 
						browser_menu->sel);
				if(browser_menu_overlay) {
					browser_menu = browser_menu_overlay;
					browser_menu_overlay = 0;
				}
				browser_do_draw();
			}	
			break;

		case 'm':
		case 'q':
			browser_menu = menu_destroy(browser_menu);
			ret |= KEY_CLICK;
			if(browser_menu != NULL) {
				browser_do_draw();
				break;
			}
			browser_exit();
			GrDestroyGC(browser_gc);
			pz_close_window(browser_wid);
			break;

		case 'r':
			if (menu_shift_selected(browser_menu, 1)) {
				menu_draw(browser_menu);
				ret |= KEY_CLICK;
			}
			break;

		case 'l':
			if (menu_shift_selected(browser_menu, -1)) {
				menu_draw(browser_menu);
				ret |= KEY_CLICK;
			}
			break;
		default:
			ret |= KEY_UNUSED;
		}
		break;
	default:
		ret |= EVENT_UNUSED;
		break;
	}
	return ret;
}

void new_browser_window(char *initial_path)
{
	/* if (initial_path) {
		chdir(initial_path);
	} */
	chdir("/sbin");
	getcwd(current_dir, sizeof(current_dir));

	browser_gc = pz_get_gc(1);
	GrSetGCUseBackground(browser_gc, GR_FALSE);
	GrSetGCBackground(browser_gc, WHITE);
	GrSetGCForeground(browser_gc, BLACK);

	browser_wid = pz_new_window(0, HEADER_TOPLINE + 2, screen_info.cols,
                                    screen_info.rows - (HEADER_TOPLINE + 2),
                                    browser_do_draw, browser_do_keystroke);

	GrSelectEvents(browser_wid, GR_EVENT_MASK_EXPOSURE |
			GR_EVENT_MASK_KEY_UP | GR_EVENT_MASK_KEY_DOWN |
			GR_EVENT_MASK_TIMER);

	browser_menu = NULL;
	browser_menu_overlay = NULL;
	browser_mscandir("./");

	GrMapWindow(browser_wid);
}

void new_exec_window(char *filename)
{
#ifdef IPOD
	int ttyfd, status, fd;
	pid_t pid;

	if((ttyfd = open("/dev/console", O_RDWR)) == -1) {
                perror("/dev/console");
		return;
	}
	if(ioctl(ttyfd, VT_ACTIVATE, 0x2)) {
		perror("child VT_ACTIVATE");
		_exit(1);
	}
	if(ioctl(ttyfd, VT_WAITACTIVE, 0x2)) {
		perror("child VT_WAITACTIVE");
		_exit(1);
	}
	
	switch(pid = vfork()) {
	case -1: /* error */
		perror("vfork");
		break;
	case 0: /* child */
		close(ttyfd);
		if(setsid() < 0) {
			perror("setsid");
			_exit(1);
		}
		close(0);
		if((fd = open("/dev/console", O_RDWR)) == -1) {
			perror("/dev/console");
			_exit(1);
		}
		if(dup(fd) == -1) {
			perror("stdin dup");
			_exit(1);
		}
		close(1);
		if(dup(fd) == -1) {
			perror("stdout dup");
			_exit(1);
		}
		close(2);
		if(dup(fd) == -1) {
			perror("stderr dup");
			_exit(1);
		}

		execl("/bin/sh", "sh", "-c", filename);
		fprintf(stderr, _("Exec failed! (Check Permissions)\n"));
		exit(1);
		break;
	default: /* parent */
		waitpid(pid, &status, 0);
		sleep(5);
		
        	if(ioctl(ttyfd, VT_ACTIVATE, 0x1)) {
			perror("parent VT_ACTIVATE");
			return;
		}
        	if(ioctl(ttyfd, VT_WAITACTIVE, 0x1)) {
			perror("parent VT_WAITACTIVE");
			return;
		}
		if(ioctl(ttyfd, VT_DISALLOCATE, 0x2)) {
			perror("VT_DISALLOCATE");
			return;
		}
		close(ttyfd);

		browser_do_draw();
		break;
	}
#else
	new_message_window(filename);
#endif /* IPOD */
}

