/*
 * Copyright (C) 2005 Courtney Cavin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "pz.h"
#include "ipod.h"
#include "mlist.h"

extern void new_contrast_window(void);
extern void new_browser_window(void);
extern void toggle_backlight(void);
extern void set_wheeldebounce(void);
extern void set_buttondebounce(void);
extern void new_font_window(menu_st *);

extern void quit_podzilla(void);
extern void poweroff_ipod(void);
extern void reboot_ipod(void);
static void change_font(void);

extern void launch_podzilla(void);
extern void launch_vipodzilla(void);
extern void launch_pokezilla(void);
extern void launch_floydzilla(void);
extern void launch_yankeezilla(void);
extern void launch_podzillati(void);

static GR_WINDOW_ID menu_wid;
static GR_GC_ID menu_gc;
static menu_st *menuz;

static char *backlight_options[] = {
	N_("Off"), N_("1 sec"), N_("2 secs"), N_("5 secs"), N_("10 secs"),
	N_("30 secs"), N_("1 min"), N_("On")
};

static item_st reset_menu[] = {
	{N_("Cancel"), NULL, SUB_MENU_PREV},
	{N_("Absolutely"), ipod_reset_settings, ACTION_MENU | SUB_MENU_PREV},
	{0}
};

static item_st appearance_menu[] = {
	{N_("Color Scheme"), colorscheme_names, OPTION_MENU, COLORSCHEME, CS_NSCHEMES },
	{N_("Decorations"), appearance_decorations, OPTION_MENU, DECORATIONS, NDECORATIONS },
	{N_("Battery Digits"), NULL, BOOLEAN_MENU, BATTERY_DIGITS },
	{N_("Display Load Average"), NULL, BOOLEAN_MENU, DISPLAY_LOAD },
	{N_("Menu Transition"), NULL, BOOLEAN_MENU, SLIDE_TRANSIT},
	{N_("Font"), change_font, ACTION_MENU},
	{ 0 }
};

static void change_font()
{
	new_font_window(menuz);
}

void menu_adjust_nschemes( int val )
{
	// '0' is the index above for "Color Scheme"
	appearance_menu[0].item_count = val;
}

static item_st settings_menu[] = {
	{N_("Contrast"), set_contrast, ACTION_MENU},
	{N_("Wheel Sensitivity"), set_wheeldebounce, ACTION_MENU},
	{N_("Button Debounce"), set_buttondebounce, ACTION_MENU},
	{N_("Backlight Timer"), backlight_options, OPTION_MENU, BACKLIGHT_TIMER, 8},
	{N_("Clicker"), NULL, BOOLEAN_MENU, CLICKER},
	{N_("Appearance"), appearance_menu, SUB_MENU_HEADER},
	{N_("Browser Path Display"), NULL, BOOLEAN_MENU, BROWSER_PATH},
	{N_("Show Hidden Files"), NULL, BOOLEAN_MENU, BROWSER_HIDDEN},
	{N_("Reset All Settings"), reset_menu, SUB_MENU_HEADER},
	{N_("Save Settings"), ipod_save_settings, ACTION_MENU},
	{N_("Load Settings"), ipod_load_settings, ACTION_MENU},
	{0}
};

static item_st reboot_menu[] = {
	{N_("Cancel"), NULL, SUB_MENU_PREV},
	{N_("Absolutely"), reboot_ipod, ACTION_MENU},
	{0}
};

static item_st turnoff_menu[] = {
	{N_("Cancel"), NULL, SUB_MENU_PREV},
	{N_("Absolutely"), poweroff_ipod, ACTION_MENU},
	{0}
};

static item_st power_menu[] = {
	{N_("Quit Podzillae"), quit_podzilla, ACTION_MENU},
	{N_("Reboot iPod"), reboot_menu, SUB_MENU_HEADER},
#ifdef NEVER /* just to show where this should go */
	{N_("Sleep iPod"), sleep, ACTION_MENU};
#endif
	{N_("Turn off iPod"), turnoff_menu, SUB_MENU_HEADER},	
	{0}
};

static item_st main_menu[] = {
	{N_("podzilla"), launch_podzilla, ACTION_MENU},
	{N_("viPodzilla"), launch_vipodzilla, ACTION_MENU},
	{N_("Poke-Zilla"), launch_pokezilla, ACTION_MENU},
	{N_("FloydZilla"), launch_floydzilla, ACTION_MENU},
	{N_("yankeezilla"), launch_yankeezilla, ACTION_MENU},
	{N_("podzilla-ti"), launch_podzillati, ACTION_MENU},
	{N_("All Podzillae"), new_browser_window, ACTION_MENU | ARROW_MENU},
	{N_("Settings"), settings_menu, SUB_MENU_HEADER},
	{N_("Power"), power_menu, SUB_MENU_HEADER},
	{0}
};

static void menu_do_draw()
{
	/* window is focused */
	if(menu_wid == GrGetFocus()) {
		pz_draw_header(menuz->title);
		menu_draw(menuz);
	}
}

static int menu_do_keystroke(GR_EVENT * event)
{
	int ret = 0;

	switch (event->type) {
	case GR_EVENT_TYPE_TIMER:
		menu_draw_timer(menuz);
		break;
	case GR_EVENT_TYPE_KEY_DOWN:
		switch (event->keystroke.ch) {
		case '\r':
		case '\n':
			menuz = menu_handle_item(menuz, menuz->sel);
			menu_do_draw();
			ret |= KEY_CLICK;
			break;
		case 'l':
			if (menu_shift_selected(menuz, -1)) {
				menu_draw(menuz);
				ret |= KEY_CLICK;
			}
			break;
		case 'r':
			if (menu_shift_selected(menuz, 1)) {
				menu_draw(menuz);
				ret |= KEY_CLICK;
			}
			break;
		case 'm':
			if(menuz->parent != NULL) {
				menuz = menu_destroy(menuz);
				menu_do_draw();
				ret |= KEY_CLICK;
			}
			break;
		case 'q':
			menu_destroy_all(menuz);
			pz_close_window(menu_wid);
			GrDestroyGC(menu_gc);
			exit(0);
			break;
		default:
			ret |= KEY_UNUSED;
			break;
		}
		break;
	default:
		ret |= EVENT_UNUSED;
		break;
	}
	return ret;
}

void new_menu_window()
{
	GrGetScreenInfo(&screen_info);

	menu_gc = pz_get_gc(1);
	GrSetGCUseBackground(menu_gc, GR_TRUE);
	GrSetGCForeground(menu_gc, BLACK);
	GrSetGCBackground(menu_gc, WHITE);

	menu_wid = pz_new_window(0, HEADER_TOPLINE + 2, screen_info.cols,
			screen_info.rows - (HEADER_TOPLINE + 2), menu_do_draw,
			menu_do_keystroke);

	GrSelectEvents(menu_wid, GR_EVENT_MASK_EXPOSURE| GR_EVENT_MASK_KEY_UP|
			GR_EVENT_MASK_KEY_DOWN | GR_EVENT_MASK_TIMER);

	menuz = menu_init(menu_wid, "podzillae", 0, 0, screen_info.cols,
			screen_info.rows - (HEADER_TOPLINE + 1), NULL,
			main_menu, ASCII | TRANSLATE);

	GrMapWindow(menu_wid);
}
